/************************************************************************************************

 Author        : ADI - Apps            www.analog.com/AduC7060

 Date          : November 2007

 File          : SPI_Master.c

 Hardware      : ADuC706x

 Description   : This SPI Master example shows how to initialise the SPI peripheral in Master mode
 			This simple example transmits 6 bytes to a Slave after a reset.
			When an XIRQ0 interrupt occurs, the Master will then read back 6 bytes from the Slave.
				

*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT17 0x20000
#define BIT22 0x400000
#define BIT25 0x2000000
#define BIT30 0x40000000

#include<aduc7060.h>
// Function Definitions
unsigned char szTxData[] = {0x41, 0x42, 0x43, 0x44, 0xd, 0x0a};	// Array to send to Slave
unsigned char ucTxCount = 0;  // Array index variable for szTxData[]
unsigned char szRxData[16];	  // Array for reading Data from Slave
unsigned char ucRxCount = 0;  // Array index variable for szRxData[]
unsigned char ucWaitingForXIRQ0 = 1; // Flag to begin reading from Slave - controlled by XIRQ0
unsigned char ucDummyRd = 0;
unsigned char UartBytesToSend = 0;
unsigned char ucTxBufferEmpty = 0;
unsigned char ucTest = 0;
unsigned int SPIMSTATUS = 0;
int main(void)
{
	unsigned char n = 0;    

	POWKEY1 = 0x1;
	POWCON0 = 0x78;		   // Set core to max CPU speed of 10.24Mhz
	POWKEY2 = 0xF4;
	GP1DAT  = BIT30;		// Configure P1.6 as an output
//	GP0DAT = BIT25;

	GP1CON = BIT0 + BIT4;  // Select UART functionality for P1.0/P1.1
		// Initialize the UART for 9600-8-N
	COMCON0 = BIT7;			// Enable access to COMDIV registers
	COMDIV0 = 0x21;
	COMDIV1 = 0x00;
	//COMDIV2 = 0x21 + BIT11; // Enable fractional divider for more accurate baud rate setting
	// Configure P0.0, P0.1,P0.2 and P0.3 for SPI mode
	COMCON0 = BIT0 + BIT1 + BIT2;
	COMIEN0 = BIT0 + BIT1;	 // Enable UART interrupts when Rx full and Tx buffer empty.
	GP0CON0 = BIT0 + BIT4 + BIT8 + BIT12; // Select SPI/I2C alternative function for P0[0...3]
	GP0KEY1 = 0x7;		   //Write to GP0KEY1
	GP0CON1	= 0x00;	   // Select SPI functionality for P0.0 to  P0.3
	GP0KEY2 = 0x13;		   //Write to GP0KEY2

	
	SPICON = BIT0 + BIT1   // Enable SPI + Enable SPI Master
		    + BIT6  // Clk pulses high at start of each bit + Initiate transfer on write to Tx FIFO
		   + BIT11 		   //Continuous transfer mode + Enable IRQ on 2 byte transfer
		  // + BIT14		   // IRQ after 1-4 bytes
		   + BIT15;
	SPIDIV  = 0x10;	   		// Select 101kHz clock rate = 0x31
	IRQCONE = BIT1;			// Rising edge of XIRQ0
	IRQEN = BIT11 + BIT12 + BIT13; 		// Enable UART + SPI + XIRQ0 interrupt

	// Begin Master Transmit sequence
	ucTxCount = 0;
	/* Test code for Tx FIFO status bits
//	SPITX = 0x55;
	SPIMSTATUS = SPISTA;
	SPITX = 0x55;
	SPIMSTATUS = SPISTA;
	SPITX = 0x55;
	SPIMSTATUS = SPISTA;
	SPITX = 0x55;
	SPIMSTATUS = SPISTA;
	SPITX = 0x55;
	SPIMSTATUS = SPISTA; 
	SPITX = szTxData[ucTxCount++];
    SPITX = szTxData[ucTxCount++];
	SPITX = szTxData[ucTxCount++];
	SPITX = szTxData[ucTxCount++];
	SPITX = szTxData[ucTxCount++];
	SPITX = szTxData[ucTxCount++];
	SPITX = szTxData[ucTxCount++];
	SPITX = szTxData[ucTxCount++]; */
	SPITX = szTxData[ucTxCount++];
	SPITX = szTxData[ucTxCount++];
	SPITX = szTxData[ucTxCount++];
	ucWaitingForXIRQ0 = 1;
	ucRxCount = 0;
		while (1)
	{
	 	
		if (ucWaitingForXIRQ0 == 0)	// Wait for XIRQ0 to trigger - cleared in XIRQ0 ISR
		{
			ucWaitingForXIRQ0 = 1;
			ucRxCount = 0;
			SPICON |= BIT12; // Flush Rx FIFO
			SPICON &= ~BIT12; 
			SPICON &= ~BIT6; // Generate Interrupt on receiving data
		//	SPICON |= BIT14 + BIT15;
			ucDummyRd = SPIRX;
		}
		if (UartBytesToSend == 1)
		{
		 	for (n = 1; n < 7 ; n++)
			{
			while (ucTxBufferEmpty != 1){}
				ucTxBufferEmpty = 0;
				COMTX = szRxData[n];
			}
			UartBytesToSend = 0;				
		}
	}
   return 0x00;
}

void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;

	unsigned char Underflow = 0;
	unsigned char ucCOMIID0 = 0;

	IRQSTATUS = IRQSTA;	   // Read off IRQSTA register
	if ((IRQSTATUS & BIT13) == BIT13)	//XIRQ0 interrupt source
	{
		ucWaitingForXIRQ0 = 0;			// Enable Read request from Slave.
		IRQCLRE = BIT13;
	}

	if ((IRQSTATUS & BIT12) == BIT12)	//If SPI Master interrupt source
	{
	   SPIMSTATUS = SPISTA;
	   if ((SPIMSTATUS & BIT4) == BIT4) // SPI Tx underflow
	   {
			Underflow = 1;
	   }
	   if ((SPIMSTATUS & BIT5) == BIT5) // If SPI Master Tx IRQ
	   {
			if (ucTxCount < 6)		   // Have max 6 bytes been sent?
			{
				SPITX = szTxData[ucTxCount++];	 // Load Tx buffer
				SPITX = szTxData[ucTxCount++];	 // Load Tx buffer
				SPITX = szTxData[ucTxCount++];	 // Load Tx buffer
			}
	   }
	   if ((SPIMSTATUS & BIT6) == BIT6) // If SPI Master Rx IRQ
	   {
			ucTest++;
			GP1DAT ^= BIT22;	// Toggle LED on Evaluation board (P1.6)
			if (ucRxCount < 7)		   // Have max 6 bytes been received?
			{
				szRxData[ucRxCount++] = SPIRX;  // Read Rx buffer
				szRxData[ucRxCount++] = SPIRX;  // Read Rx buffer
				szRxData[ucRxCount++] = SPIRX;  // Read Rx buffer
			}
			else
			{
				ucRxCount = 0;
				/* Test for SPI RX overwrite - check values in array with SPICON.8 set & cleared  
				SPIMSTATUS = SPISTA;
				szRxData[ucRxCount++] = SPIRX;
				SPIMSTATUS = SPISTA;
				szRxData[ucRxCount++] = SPIRX;
			//	SPICON |= BIT12; // Flush Rx FIFO
			//	SPICON &= ~BIT12;
				SPIMSTATUS = SPISTA;
				szRxData[ucRxCount++] = SPIRX;
				SPIMSTATUS = SPISTA;
				szRxData[ucRxCount++] = SPIRX;
				SPIMSTATUS = SPISTA; 
				*/
				UartBytesToSend = 1;
			}
			GP1DAT ^= BIT22;	// Toggle LED on Evaluation board (P1.6)
	   }
	}
	if ((IRQSTATUS & BIT11) == BIT11)	//UART interrupt source
	{
		ucCOMIID0 = COMIID0;
		if ((ucCOMIID0 & 0x2) == 0x2)	  // Transmit buffer empty
		{
		  ucTxBufferEmpty = 1;
		}
	}
}


